-- @author peter.schaefer@gmail.com
---local arg={...};

local mode= arg[1]

if mode==nil then
	print("Program needs two arguments, mode in SSSOn, SSSOff, AutoDocOn, AutoDocOff, Status, and the path %APPDATA%.")
	os.exit(200)
end

local path= arg[2]
if path==nil then
	path=""
else
	path= path.."\\SpieleEntwicklungsKombinat\\Paraworld\\"
end

function exists(n)
	local f=io.open(n)
	if f then io.close(f) end
	return f~=nil
end

function copyfiletofile(a,b)
    local fa = io.open(a, "r")
	if fa==nil then return false end

    local fb = io.open(b, "w")
	if fb==nil then return false end

    fb:write(fa:read("*all"))

    fb:close()
    fa:close()
	return true
end

local original=path.."Settings.cfg"
local backup=path.."Settings_SSSS_backup.cfg"
local target=path.."Settings.cfg"

if not exists(backup) then
	if not copyfiletofile(original,backup) then
		print "Could not create backup file!"
		os.exit(101)
	end
end

local fa = io.open(original,"r")

if not fa then
	print("Input file not found!")
	os.exit(102)
end

function prettyprint(fh,tree,indent)
	local previous_label= nil
	local previous_value= nil

	local label
	local value
	local values= tree[1]
	local orderings= tree[2]
	for ordering,label in ipairs(orderings) do
		local value= values[label]
		if type(value) == 'table' then
			if table.maxn(value[2])~=0 then
				fh:write(indent..label.." ".."{\n")
				prettyprint(fh,value,indent.."\t")
				fh:write(indent.."}\n")
			end
		elseif value ~=nil then
			-- don't print duplicates
			if previous_label~=label then
				fh:write(indent..label.." = '"..value.."'\n")
				previous_label= label
				previous_value= value
			end
		end
	end
end

print("Reading table")
root={{},{}}
current=root
path={current}

local parsestate="seeklabel"
local label
local value
local c=fa:read(1)
while c~=nil do
	if parsestate=="seeklabel" then
		if c==" " or c=="\t" or c=="\r" or c=="\n" then
			-- do nothing
		elseif c=="{" then
			print("unexpected "..c)
			os.exit(250)
		elseif c=="}" then
		    --pop a subtree
			table.remove(path)
			current= path[table.maxn(path)]
			parsestate= "seeklabel"
		elseif c=="=" then
			print("unexpected "..c)
			os.exit(250)
		elseif c=="'" then
			print("unexpected "..c)
			os.exit(250)
		else
			label= c
			parsestate="label"
		end
	elseif parsestate=="label" then
		if c==" " or c=="\t" or c=="\r" or c=="\n" then
			parsestate="seekequals"
		elseif c=="{" then
		    --push a subtree
			local tree={{},{}}
			current[1][label]= tree
			table.insert(current[2],label)
			table.insert(path,tree)
			current= tree
			parsestate= "seeklabel"
		elseif c=="}" then
			print("unexpected "..c)
			os.exit(250)
		elseif c=="=" then
			value=nil
			parsestate= "seekvalue"
		elseif c=="'" then
			print("unexpected "..c)
			os.exit(250)
		else
			label= label..c
		end
	elseif parsestate=="seekequals" then
		if c==" " or c=="\t" or c=="\r" or c=="\n" then
			--do nothing
		elseif c=="{" then
		    --push a subtree
			local tree={{},{}}
			current[1][label]= tree
			table.insert(current[2],label)
			table.insert(path,tree)
			current= tree
			parsestate= "seeklabel"
		elseif c=="}" then
			print("unexpected "..c)
			os.exit(250)
		elseif c=="=" then
			value=nil
			parsestate="seekvalue"
		else
			print("unexpected "..c)
			os.exit(250)
		end
	elseif parsestate=="seekvalue" then
		if c==" " or c=="\t" or c=="\r" or c=="\n" then
			--do nothing
		elseif c=="{" then
			print("unexpected "..c)
			os.exit(250)
		elseif c=="}" then
			print("unexpected "..c)
			os.exit(250)
		elseif c=="=" then
			print("unexpected "..c)
			os.exit(250)
		elseif c=="'" then
			value=""
			parsestate="value"
		else
			print("unexpected "..c)
			os.exit(250)
		end
	elseif parsestate=="value" then
		if c=="'" then
			current[1][label]= value
			table.insert(current[2],label)
			parsestate="seeklabel"
		else
			value= value..c
		end
	else
	end
	c=fa:read(1)
end

io.close(fa)

function createpath(current,...)
	local arg={...}
	local i, dir
	for i,dir in ipairs(arg) do
		if current[1][dir] == nil then
			current[1][dir] = {{},{}}
			table.insert(current[2],1,dir)
		elseif type(current[1][dir]) ~= 'table' then
			if type(current[1][dir]) == 'string' then
				if current[1][dir] == "" then
					print("Fixing bad path") -- fix bad output of v0.4.1
					current[1][dir] = {{},{}}
					--table.insert(current[2],1,dir)
				else
					print("Not fixing bad path, "..dir..", it contains a value.")
					os.exit(104)
				end
			else
				print("Unexpected value type: ",type(current[1][dir]))
				os.exit(105)
			end
		end
		current= current[1][dir]
	end
end

function table.findindex(t,value)
	for ix,val in ipairs(t) do
		if val==value then return ix end
	end
	return 0
end

function containspath(current,...)
	local arg={...}
	local i, dir
	for i,dir in ipairs(arg) do
		if current[1][dir] == nil then
			return false
		end
		current= current[1][dir]
	end
	return true
end

if mode=="SSSOn" then
	print("Modifying table, switching SSS on")

	createpath(root,"Root","Pest","Server")

	root[1]["Root"][1]["Pest"][1]["Server"][1]["UseUslCPP"] = "0"
	table.insert(root[1]["Root"][1]["Pest"][1]["Server"][2],1,"UseUslCPP")

	createpath(root,"Root","Pest","Global")

	root[1]["Root"][1]["Pest"][1]["Global"][1]["UslBinary"] = "0"
	table.insert(root[1]["Root"][1]["Pest"][1]["Global"][2],1,"UslBinary")

elseif mode=="SSSOff" then
	print("Modifying table, switching SSS off")

	local index

	if containspath(root,"Root","Pest","Server") then
		root[1]["Root"][1]["Pest"][1]["Server"][1]["UseUslCPP"] = nil
		index= table.findindex(root[1]["Root"][1]["Pest"][1]["Server"][2],"UseUslCPP")
		if index>0 then
			table.remove(root[1]["Root"][1]["Pest"][1]["Server"][2],index)
		end
	end

	if containspath(root,"Root","Pest","Global") then
		root[1]["Root"][1]["Pest"][1]["Global"][1]["UslBinary"] = nil
		index= table.findindex(root[1]["Root"][1]["Pest"][1]["Global"][2],"UslBinary")
		if index>0 then
			table.remove(root[1]["Root"][1]["Pest"][1]["Global"][2],index)
		end
	end

elseif mode=="AutoDocOn" then
	print("Modifying table, switching AutoDoc on")

	createpath(root,"Root","Server")

	root[1]["Root"][1]["Server"][1]["CreateUSLHtmlAutoDoc"] = "1"
	table.insert(root[1]["Root"][1]["Server"][2],1,"CreateUSLHtmlAutoDoc")

	createpath(root,"Root","Client")

	root[1]["Root"][1]["Client"][1]["CreateUSLHtmlAutoDoc"] = "1"
	table.insert(root[1]["Root"][1]["Client"][2],1,"CreateUSLHtmlAutoDoc")

	createpath(root,"Root","AI")

	root[1]["Root"][1]["AI"][1]["CreateUSLHtmlAutoDoc"] = "1"
	table.insert(root[1]["Root"][1]["AI"][2],1,"CreateUSLHtmlAutoDoc")

elseif mode=="AutoDocOff" then
	print("Modifying table, switching AutoDoc off")

	local index

	if containspath(root,"Root","Server") then
		root[1]["Root"][1]["Server"][1]["CreateUSLHtmlAutoDoc"] = nil
		index= table.findindex(root[1]["Root"][1]["Server"][2],"CreateUSLHtmlAutoDoc")
		if index>0 then
			table.remove(root[1]["Root"][1]["Server"][2],index)
		end
	end

	if containspath(root,"Root","Client") then
		root[1]["Root"][1]["Client"][1]["CreateUSLHtmlAutoDoc"] = nil
		index= table.findindex(root[1]["Root"][1]["Client"][2],"CreateUSLHtmlAutoDoc")
		if index>0 then
			table.remove(root[1]["Root"][1]["Client"][2],index)
		end
	end

	if containspath(root,"Root","AI") then
		root[1]["Root"][1]["AI"][1]["CreateUSLHtmlAutoDoc"] = nil
		index= table.findindex(root[1]["Root"][1]["AI"][2],"CreateUSLHtmlAutoDoc")
		if index>0 then
			table.remove(root[1]["Root"][1]["AI"][2],index)
		end
	end

elseif mode=="Status" or mode=="Statuscode" then

	local status= 0

	local Root_Pest_Server_UseUslCPP= "On"
	if containspath(root,"Root","Pest","Server") then
		if root[1]["Root"][1]["Pest"][1]["Server"][1]["UseUslCPP"] == "0" then
			Root_Pest_Server_UseUslCPP= "Off"
			status= status + 1
		end
	end

	local Root_Pest_Global_UslBinary= "On"
	if containspath(root,"Root","Pest","Global") then
		if root[1]["Root"][1]["Pest"][1]["Global"][1]["UslBinary"] == "0" then
			Root_Pest_Global_UslBinary= "Off"
			status= status + 2
		end
	end

	local Root_Server_CreateUSLHtmlAutoDoc= "Off"
	if containspath(root,"Root","Server") then
		index= table.findindex(root[1]["Root"][1]["Server"][2],"CreateUSLHtmlAutoDoc")
		if root[1]["Root"][1]["Server"][1]["CreateUSLHtmlAutoDoc"] == "1" then
			Root_Server_CreateUSLHtmlAutoDoc= "On"
			status= status + 4
		end
	end

	local Root_Client_CreateUSLHtmlAutoDoc= "Off"
	if containspath(root,"Root","Client") then
		if root[1]["Root"][1]["Client"][1]["CreateUSLHtmlAutoDoc"] == "1" then
			Root_Client_CreateUSLHtmlAutoDoc=" On"
			status= status + 8
		end
	end

	local Root_AI_CreateUSLHtmlAutoDoc= "Off"
	if containspath(root,"Root","AI") then
		if root[1]["Root"][1]["AI"][1]["CreateUSLHtmlAutoDoc"] == "1" then
			Root_AI_CreateUSLHtmlAutoDoc= "On"
			status= status + 16
		end
	end

	local Serversidescripting
	if Root_Pest_Server_UseUslCPP == "Off" and Root_Pest_Global_UslBinary == "Off" then
		Serversidescripting= "On"
	elseif Root_Pest_Server_UseUslCPP ~= Root_Pest_Global_UslBinary then
		Serversidescripting= "Mixed"
	else
		Serversidescripting= "Off"
	end

	print("Server side scripting= ",Serversidescripting)
	print("Root.Pest.Server.UseUslCPP= ",Root_Pest_Server_UseUslCPP)
	print("Root.Pest.Global.UslBinary= ",Root_Pest_Global_UslBinary)

	print("Root.Server.CreateUSLHtmlAutoDoc= ",Root_Server_CreateUSLHtmlAutoDoc)
	print("Root.Client.CreateUSLHtmlAutoDoc= ",Root_Client_CreateUSLHtmlAutoDoc)
	print("Root.AI.CreateUSLHtmlAutoDoc= ",Root_AI_CreateUSLHtmlAutoDoc)

	if mode=="Statuscode" then
		os.exit(status)
	else
		os.exit(0)
	end
elseif mode==nil then
	print("Program needs two arguments, mode in SSSOn, SSSOff, AutoDocOn, AutoDocOff, Status and the path %APPDATA%.")
	os.exit(201)
else
	print("Unrecognized mode ", mode)
	os.exit(202)
end

print("Outputting table")
local fb = io.open(target,"w")

if not fb then
	print("Output file cannot be created!")
	os.exit(103)
end

prettyprint(fb,root,"")

io.close(fb)

os.exit(0)
